package com.brilliance.isc;

import org.mybatis.spring.annotation.MapperScan;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.boot.web.servlet.ServletComponentScan;
import org.springframework.boot.web.servlet.support.SpringBootServletInitializer;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.netflix.hystrix.EnableHystrix;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.util.StopWatch;
import org.springframework.web.WebApplicationInitializer;

import java.io.IOException;
import java.net.URL;
import java.util.Map;
import java.util.Objects;

@ComponentScan(basePackages = {"com.brilliance.isc","com.brilliance.mda"})
//扫描mybatisMapper接口类，注意：包需细化到尽可能小的范围
@MapperScan(basePackages = {"com.brilliance.isc.**.mapper","com.brilliance.isc.mda.dao"})
@SpringBootApplication
@EnableFeignClients
@ServletComponentScan(basePackages = {"com.brilliance.isc"})
@EnableHystrix
@EnableDiscoveryClient
public class App  extends SpringBootServletInitializer implements WebApplicationInitializer {
	private static Logger logger = LoggerFactory.getLogger(App.class);

	@Override
	protected SpringApplicationBuilder configure(SpringApplicationBuilder application) {
		return application.sources(App.class);
	}

	public static void main( String[] args )throws IOException,
			ClassNotFoundException{
		StopWatch watch = new StopWatch();
		watch.start();
		init();
		ConfigurableApplicationContext context = SpringApplication.run(App.class, args);
		//...
		context.registerShutdownHook();

		watch.stop();
		logger.info("----->>>>>App startup takes " + watch.getTotalTimeMillis() + " ms.<<<<<-----");
	}


	private static void init() {
		//应用必需
		setPodName();
		setLog4j2Path();
	}

	//用于指定云环境生成带POD名的日志文件
	private static void setPodName() {
		Map<String, String> map = System.getenv();
		if(null == map.get("SSTF_DIR")) {
			System.setProperty("SSTF_DIR", "/sstf");
		}
		if (null != map.get("MY_POD_NAME")) {
			System.setProperty("MY_POD_NAME", map.get("MY_POD_NAME"));
		} else {
			System.setProperty("MY_POD_NAME", "default");
			System.setProperty("MY_POD_IP", "127.0.0.1");
			logger.error("请在环境变量或jvm启动参数上配置参数MY_POD_NAME,用于指定云环境生成带POD名的日志文件");
		}
	}
	//设置log4j2.xml路径
	private static void setLog4j2Path() {
		URL url = App.class.getResource("");
		String protocol = url.getProtocol();
		if("file".equals(protocol)){
			System.setProperty("logging.config","classpath:log4j2.xml");
		}
	}

	//加解密工具类
	private static void encryptionAndDecryption() {
		if(isStartupFromJar()) {
			return ;
		}
		//String encryptPwd = JasyptUtils.encryptPwd(secretKey, "abc");
		String encryptPwd = "EbfYkitulv73I2p0mXI50JMXoaxZTKJ7";
		logger.error("加密后-->" + encryptPwd);
		//String decyptPwd = JasyptUtils.decyptPwd(secretKey, "WQMRNxQay4JSTkYMSm86FA==");
		String decyptPwd = "WQMRNxQay4JSTkYMSm86FA";
		logger.error("解密后-->" + decyptPwd);
		//使用时需要加ENC()
		//加密生成的密文每次不相同, 都可以正常解密, 放心使用
		//启动参数或环境变量使用方式 java -jar *.jar --key='ENC(s9i3k93kj3j2ksud76eh342)'
	}

	private static <T> boolean isStartupFromJar() {
		String protocol = App.class.getResource("").getProtocol();
		if (Objects.equals(protocol, "jar")) {
			return true;
		} else if (Objects.equals(protocol, "file")) {
			return false;
		}
		return true;
	}

}
