package com.brilliance.isc.funds.common.component;

import com.brilliance.isc.bo.*;
import com.brilliance.isc.bo.model.*;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.component.PtsptaToolComponent;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.exception.CommonServiceException;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.transaction.vo.BaseTransactionVo;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.util.Utils;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.mda.dao.*;
import com.brilliance.isc.vo.TrncodVo;
import com.brilliance.mda.runtime.mda.Constants;
import com.brilliance.mda.runtime.mda.IStream;
import com.brilliance.mda.runtime.mda.impl.Argument;
import com.brilliance.mda.runtime.mda.impl.StreamImpl;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.brilliance.mda.runtime.mda.util.Streams;
import com.brilliance.mda.runtime.mda.util.Strings;
import com.google.common.collect.ImmutableBiMap;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.util.*;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: CrToolComponent
 * @description: 清算 crtmod crdlod
 * @author: huangshunlin
 * @create: 2024-10-16 20:24
 */
@Component
public class CrToolComponent {

    @Resource
    private CrdMapper crdMapper;

    @Resource
    private PtsptaToolComponent ptsptaToolComponent;

    @Resource
    private CbsmodService cbsmodService;

    @Resource
    private SysmodService sysmodService;

    @Resource
    private TrnmodService trnmodService;

    @Resource
    private BchMapper bchMapper;


    @Resource
    private CpdMapper cpdMapper;

    @Resource
    private CptMapper cptMapper;

    @Resource
    private CndMapper cndMapper;

    @Resource
    private BcdMapper bcdMapper;

    @Resource
    private BctMapper bctMapper;

    @Resource
    private BrdMapper brdMapper;

    @Resource
    private BrtMapper brtMapper;

    @Resource
    private LidMapper lidMapper;

    @Resource
    private LitMapper litMapper;

    @Resource
    private GidMapper gidMapper;

    @Resource
    private GitMapper gitMapper;

    public Crdgrp getCrdgrpByInr(String inr) {
        Crd rec = new Crd();
        rec.setInr(inr);
        rec = crdMapper.selectByPrimaryKey(rec);
        if (Objects.isNull(rec)) {
            throw new CommonServiceException("主键" + inr + "找不到Crd数据");
        }
        Crdgrp crdgrp = new Crdgrp();
        crdgrp.reset();
        crdgrp.setRec(rec);
        //获取cbs金额信息
        Date date = Dates.add(Utils.getMaxDate(), -1);
        cbsmodService.cbsGetBalance(crdgrp.getCbs().getMax(), "CRD", inr, "MAXSUM", "AMT1", date);
        //获取ptspta信息
        storePtsToGrp(crdgrp, inr, "CRD");
        return crdgrp;
    }


    public void storePtsToGrp(Object grpObject, String objInr, String objType) {
        //找ptspta数据
        Map<String, PtsptaVo> maps = ptsptaToolComponent.getPtsptaList(objInr, objType);
        if (maps.size() > 0) {
            maps.keySet().forEach(key -> {
                PtsptaVo ptsptaVo = maps.get(key);
                if (key.equalsIgnoreCase("INT")) {
                    key = "tmpint";
                }
                Utils.setFieldValue(grpObject, key.toLowerCase(), ptsptaVo);
            });
        }
    }

    public void collectCbsMap(TransactionServiceSet set, String prefix, Cracbs cbs, String... cbsNames) {
        Map<String, Cbb> cbsMap = set.getCbsMap();
        if (cbsMap == null) {
            cbsMap = new HashMap<>();
        }
        for (String name : cbsNames) {
            Cbb cbb = Utils.getFieldValue(cbs, name);
            if (cbb != null) {
                cbsMap.put(prefix.toUpperCase() + name.toUpperCase(), cbb);
            }
        }
        set.setCbsMap(cbsMap);
    }


    public void collectPtsList(TransactionServiceSet set, Crdgrp crdgrp, String... ptsNames) {
        if (ptsNames == null) {
            //找所有的ptspta对象
            Field[] fields = crdgrp.getClass().getDeclaredFields();
            for (Field field : fields) {
                field.setAccessible(true);
                if (PtsptaVo.class.equals(field.getType())) {
                    PtsptaVo ptspta = Utils.getFieldValue(crdgrp, field.getName());
                    if (ptspta != null) {
                        set.addPtspta(transferFrom(ptspta));
                    }
                }
            }
        } else {
            for (String name : ptsNames) {
                PtsptaVo ptspta = Utils.getFieldValue(crdgrp, name);
                if (ptspta != null && ptspta.getPts() != null && StringUtils.isNotEmpty(ptspta.getPts().getRol())) {
                    set.addPtspta(transferFrom(ptspta));
                }
            }
        }
    }


    /**
     * 根据STL结构的ptspta转化成抽象组件的ptspta，结构不一样
     *
     * @param ptsptas
     * @return List<com.brilliance.common.vo.PtsptaVo>
     */
    protected PtsptaVo transferFrom(PtsptaVo ptsptas) {
        PtsptaVo ptspta = ptsptas;
        ptspta.setRol(ptsptas.getPts().getRol());
        ptspta.setPtsinr(ptsptas.getPts().getInr());
        ptspta.setNam(ptsptas.getPts().getNam());
        ptspta.setPtyinr(ptsptas.getPts().getPtyinr());
        ptspta.setPtainr(ptsptas.getPts().getPtainr());
        ptspta.setExtkey(ptsptas.getPts().getExtkey());
        ptspta.setDftdsp(ptsptas.getPts().getDftdsp());
        ptspta.setDftcur(ptsptas.getPts().getDftcur());
        ptspta.setDftact(ptsptas.getPts().getDftact());
        ptspta.setDftfeecur(ptsptas.getPts().getDftfeecur());
        ptspta.setDftactptainr(ptsptas.getPts().getDftactptainr());
        ptspta.setDftselrol(ptsptas.getPts().getDftsetrol());
        ptspta.setGlggrpflg(ptsptas.getPts().getGlggrpflg());
        ptspta.setRef(ptsptas.getPts().getRef());
        ptspta.setPtytyp(ptsptas.getPtytyp());
        ptspta.setPtyextkey(ptsptas.getPtyextkey());
        ptspta.setAdrblk(ptsptas.getPts().getAdrblk());
        return ptspta;
    }


    public String getCrdInrByOwnref(String ownref) {
        Crd crd = crdMapper.selectByOwnref(ownref);

        if (crd != null && StringUtils.isNotEmpty(crd.getInr())) {
            return crd.getInr();
        } else {
            return null;
        }

    }


    public void isFXTrnAllowed(Crdgrp crdgrp, TrncodVo trncodVo) {


    }


    /**
     * source:crtmod.0000.script
     */
    public void getref(Crdgrp crdgrp) {
        //! Set reference number, if not already set (usually called in transaction`s MTABUT\\SAV Event)
        //>>>ZL: FRA000052  相关交易增加按钮GETREF---
        //<<<ZL: END----
        /**
         $Length = 6
         **/
        int length = 5;
        // modify for hangzhou
        String tbl = NULLSTR;
        switch (crdgrp.getRec().getClrtyp()) {
            case "I":
                if (MdaUtils.compareTo(crdgrp.getRec().getTrntyp(), "") == 0) {
                    tbl = "CM";
                } else {
                    tbl = "CI";
                }
                break;
            case "O":
                tbl = "CO";
                break;
        }
        if (MdaUtils.isEmpty(crdgrp.getRec().getOwnref())) {
            crdgrp.getRec().setOwnref(sysmodService.getRef(tbl, length));
        }

    }

    /**
     * source:crtmod.0001.script
     */

    public void sav(Crdgrp crdgrp) {
        // saves contract including texts. CBS resp. of transaction
        trnmodService.trnStoreContract(crdgrp);

    }

    /**
     * source:crtmod.0002.script
     */
    public void setObjNam(String objref, String nam) {
        //Setting OBJNAM. If the setting of OBJREF is not standard, here you can change it.
//        ctx.absGet(Sysmod.class).setObjnam(NULLSTR);
//        ctx.absGet(Sysmod.class).setObjnam(objref + "/" + nam);

    }

    /**
     * source:crtmod.0003.script
     */
    public void saveClearingEntry(Crtmod crtmod, TrnmodVo trnmodVo, BaseTransactionVo bo) {
        trnmodVo.reset();
        crtmod.reset();
        // 根据AutoRegistration创造
        IStream parstm = crtmod.getParstm();
        if (MdaUtils.isEmpty(parstm)) {
            return;
        }
        //crdgrp
        Crdgrp argGrp = Utils.getFieldValue(bo,"crdgrp");
        String argGrpPath = Strings.getLine(parstm, 1);
        String argFrm = Strings.getLine(parstm, 2);
        String argTxt = Strings.getLine(parstm, 3);
        Crd rec = argGrp.getRec();
        // 只有汇出清算CRTCRO交易才会需要指定SMHINR
        if (MdaUtils.compareTo(argFrm, "CRTCRO") == 0) {
            if (MdaUtils.isEmpty(crtmod.getSmhinr())) {
                return;
            }
        }
        String recRef = rec.getOwnref();
        Trn trn = trnmodVo.getTrn();
        // 不新增SMH记录
        // SPT指向需要清算的MT202那条SMH
        Spt spt = new Spt();
        crtmod.setCrtspt(spt);
        crtmod.getCrtspt().setFrm(argFrm);
        crtmod.getCrtspt().setTxt(argTxt);
        crtmod.getCrtspt().setObjnam(Strings.getText("自动清分. $1 $2", recRef, rec.getNam()));
        crtmod.getCrtspt().setSmhinr(crtmod.getSmhinr());
        crtmod.getCrtspt().setOwnref(recRef);
        crtmod.getCrtspt().setObjtyp("CRD");
        crtmod.getCrtspt().setObjinr(Utils.recGetInr(rec));
        crtmod.getCrtspt().setOwnusr(trn.getUsr());
        Argument<String> pathBox = new Argument<String>("", crtmod.getCrtspt().getOwnusg());
        sysmodService.getTrnUSG(pathBox);

        crtmod.getCrtspt().setOwnusg(pathBox.value);

        if (MdaUtils.isEmpty(crtmod.getCrtspt().getOwnusg())) {
            crtmod.getCrtspt().setOwnusg(trn.getUsg());
        }
        crtmod.getCrtspt().setHldflg("R");
        crtmod.getCrtspt().setCreobjtyp("TRN");
        crtmod.getCrtspt().setCreobjinr(trn.getInr());
        trnmodService.trnORDSetSPTAUT(trnmodVo, crtmod.getCrtspt());
        // 自动提交到清算中心
        //增加实体信息
        String ety = SettleContext.getUserEty().getExtkey();
        String bchInr = "";
        String sql = "SELECT INR FROM BCH WHERE LEV = '1' AND CLRFLG = 'X' and etyextkey='" + ety + "'";
        List<Bch> bchList = bchMapper.dyncRead(ImmutableBiMap.of("sql", sql));
        if (CollectionUtils.isNotEmpty(bchList)) {
            bchInr = bchList.get(0).getInr();
        }

        crtmod.getCrtspt().setBchkeyinr(bchInr);
        crtmod.getCrtspt().setBranchinr(bchInr);
        IStream sptStm = new StreamImpl();
        Streams.streamClear(sptStm);
//        Systems.setContext("SAVEDATASKIPEMPTY", true);
//        String argMod = MdaDriver.getModuleInfo(argGrp, tdModuleInfoClass);
        String argMod = "CRDGRP";
        Utils.saveData(argGrp.getRec(), sptStm, "", argMod + "\\");
//        Systems.setContext("SAVEDATASKIPEMPTY", false);
        IStream bakStm = new StreamImpl();
        Streams.streamClear(bakStm);
        Streams.streamSet(bakStm, sptStm);
        sysmodService.sptSaveEntry(crtmod.getCrtspt(), argGrp.getRec(), Constants.SptWaitingRegistration, bakStm, "");

    }

    /**
     * source:crtmod.0004.script
     */
    public void registerClearingEntry(Crtmod crtmod, String argGrpPath, String argFrm, String argTxt) {
        IStream parstm = crtmod.getParstm();
        Streams.streamClear(parstm);
        Streams.streamInsert(parstm, 0, argGrpPath);
        Streams.streamInsert(parstm, 0, argFrm);
        Streams.streamInsert(parstm, 0, argTxt);

    }


    public Cpdgrp getCpdgrpByInr(String inr) {
        Cpd cpd = new Cpd();
        cpd.setInr(inr);
        cpd = cpdMapper.selectByPrimaryKey(cpd);
        if (cpd == null) {
            throw new CommonServiceException("主键" + inr + "找不到CPD数据");
        }
        Cpdgrp cpdgrp = new Cpdgrp();
        cpdgrp.reset();
        cpdgrp.setRec(cpd);
        Cpt cpt = new Cpt();
        cpt.setInr(inr);
        cpt = cptMapper.selectByPrimaryKey(cpt);
        cpdgrp.setBlk(cpt);
        //获取cbs金额信息
        Date date = Dates.add(Utils.getMaxDate(), -1);
        cbsmodService.cbsGetBalance(cpdgrp.getCbs().getMax(), "CPD", inr, "MAXSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(cpdgrp.getCbs().getNom1(), "CPD", inr, "NOMSUM", "AMT1", date);
        //获取ptspta信息
        storePtsToGrp(cpdgrp, inr, "CPD");
        //获取cnd信息
        getCndInfo(cpdgrp, inr, "CPD");
        return cpdgrp;
    }

    public void getCndInfo(Cpdgrp cpdgrp, String objInr, String objType) {
        List<Cnd> cnds = cndMapper.selectByObjtypAndObjinr(objType, objInr);
        if (!MdaUtils.isEmpty(cnds)) {
            cpdgrp.setPyebchkey(cnds.get(0).getPyebchkey());
            cpdgrp.setPyebchnam(cnds.get(0).getPyebchnam());
        }
    }

    public Bcdgrp getBcdgrpByInr(String inr) {
        Bcd rec = new Bcd();
        rec.setInr(inr);
        rec = bcdMapper.selectByPrimaryKey(rec);
        if (rec == null) {
            throw new CommonServiceException("主键" + inr + "找不到BCD数据");
        }
        String objtyp = "BCD";
        Bcdgrp bcdgrp = new Bcdgrp();
        bcdgrp.reset();
        bcdgrp.setRec(rec);
        Bct blk = new Bct();
        blk.setInr(inr);
        blk = bctMapper.selectByPrimaryKey(blk);
        bcdgrp.setBlk(blk);
        //获取cbs金额信息
        Date date = Dates.add(Utils.getMaxDate(), -1);
        cbsmodService.cbsGetBalance(bcdgrp.getCbs().getMax(), objtyp, inr, "MAXSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(bcdgrp.getCbs().getOpn1(), objtyp, inr, "OPN", "AMT1", date);
        //获取ptspta信息
        storePtsToGrp(bcdgrp, inr, objtyp);
        return bcdgrp;
    }

    public Brdgrp getBrdgrpByInr(String inr) {
        Brd rec = new Brd();
        rec.setInr(inr);
        rec = brdMapper.selectByPrimaryKey(rec);
        if (rec == null) {
            throw new CommonServiceException("主键" + inr + "找不到BRD数据");
        }
        String objtyp = "BRD";
        Brdgrp brdgrp = new Brdgrp();
        brdgrp.reset();
        brdgrp.setRec(rec);
        Brt blk = new Brt();
        blk.setInr(inr);
        blk = brtMapper.selectByPrimaryKey(blk);
        brdgrp.setBlk(blk);
        //获取cbs金额信息
        Date date = Dates.add(Utils.getMaxDate(), -1);
        cbsmodService.cbsGetBalance(brdgrp.getCbs().getMax(), objtyp, inr, "MAXSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(brdgrp.getCbs().getOpn1(), objtyp, inr, "OPN", "AMT1", date);

        //getAamCbsPerDate  MAX2/OPN2
        cbsmodService.cbsGetBalance(brdgrp.getCbs().getMax2(), objtyp, inr, "MAXSUM", "AMT2", date);
        cbsmodService.cbsGetBalance(brdgrp.getCbs().getOpn2(), objtyp, inr, "OPN", "AMT2", date);
        //获取ptspta信息
        storePtsToGrp(brdgrp, inr, objtyp);
        return brdgrp;
    }

    public Lidgrp getLidgrpByInr(String inr) {
        Lid rec = new Lid();
        rec.setInr(inr);
        rec = lidMapper.selectByPrimaryKey(rec);
        if (rec == null) {
            throw new CommonServiceException("主键" + inr + "找不到LID数据");
        }
        String objtyp = "LID";
        Lidgrp lidgrp = new Lidgrp();
        lidgrp.reset();
        lidgrp.setRec(rec);
        Lit blk = new Lit();
        blk.setInr(inr);
        blk = litMapper.selectByPrimaryKey(blk);
        lidgrp.setBlk(blk);
        //获取cbs金额信息
        Date date = Dates.add(Utils.getMaxDate(), -1);
        cbsmodService.cbsGetBalance(lidgrp.getCbs().getMax(), objtyp, inr, "MAXSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(lidgrp.getCbs().getNom1(), objtyp, inr, "NOMSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(lidgrp.getCbs().getOpn1(), objtyp, inr, "OPN", "AMT1", date);

        //getAamCbsPerDate  MAX2/OPN2
        cbsmodService.cbsGetBalance(lidgrp.getCbs().getMax2(), objtyp, inr, "MAXSUM", "AMT2", date);
        cbsmodService.cbsGetBalance(lidgrp.getCbs().getOpn2(), objtyp, inr, "OPN", "AMT2", date);
        //获取ptspta信息
        storePtsToGrp(lidgrp, inr, objtyp);
        return lidgrp;
    }

    public Gidgrp getGidgrpByInr(String inr) {
        Gid rec = new Gid();
        rec.setInr(inr);
        rec = gidMapper.selectByPrimaryKey(rec);
        if (rec == null) {
            throw new CommonServiceException("主键" + inr + "找不到GID数据");
        }
        String objtyp = "GID";
        Gidgrp gidgrp = new Gidgrp();
        gidgrp.reset();
        gidgrp.setRec(rec);
        Git blk = new Git();
        blk.setInr(inr);
        blk = gitMapper.selectByPrimaryKey(blk);
        gidgrp.setBlk(blk);
        //获取cbs金额信息
        Date date = Dates.add(Utils.getMaxDate(), -1);
        cbsmodService.cbsGetBalance(gidgrp.getCbs().getMax(), objtyp, inr, "MAXSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(gidgrp.getCbs().getOpn1(), objtyp, inr, "OPN", "AMT1", date);
        cbsmodService.cbsGetBalance(gidgrp.getCbs().getCnf(), objtyp, inr, "CNFSUM", "AMT1", date);

        //getAamCbsPerDate  MAX2/OPN2
        cbsmodService.cbsGetBalance(gidgrp.getCbs().getMax2(), objtyp, inr, "MAXSUM", "AMT2", date);
        cbsmodService.cbsGetBalance(gidgrp.getCbs().getOpn2(), objtyp, inr, "OPN", "AMT2", date);
        //获取ptspta信息
        storePtsToGrp(gidgrp, inr, objtyp);
        return gidgrp;
    }

}
