package com.brilliance.isc.funds.crtcri.service.impl;

import com.brilliance.isc.bo.model.Crdgrp;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.vo.BaseTransactionVo;
import com.brilliance.isc.common.util.Utils;
import com.brilliance.isc.funds.crtcri.service.CrtpService;
import com.brilliance.isc.mda.dao.PublicMapper;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.google.common.collect.ImmutableBiMap;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @program: isc-funds
 * @ClassName: CrtpServiceImpl
 * @description: crtp 清算 公共方法
 * @author: huangshunlin
 * @create: 2024-10-22 11:25
 */
@Service
public class CrtpServiceImpl implements CrtpService {

    @Resource
    private SysmodService sysmodService;

    @Resource
    private PublicMapper publicMapper;

    /**
     * source:crtp.0002.script
     */
//    @Default(target="crdgrp.rec.ownusr",order=1000)
    public void defaultCrdgrpRecOwnusrN1000(BaseTransactionVo bo) {
        Crdgrp crdgrp = Utils.getFieldValue(bo, "crdgrp");
        if (Objects.isNull(crdgrp)) {
            return;
        }
        //! Set OWNUSR if not already set
        if (MdaUtils.isEmpty(crdgrp.getRec().getOwnusr())) {
            crdgrp.getRec().setOwnusr(SettleContext.getUserSession().getUsr().getExtkey());
        }
    }


    /**
     * source:crtp.0007.script
     */
//    @Default(value={"crdgrp.rec.clrtyp","crdgrp.rec.trntyp"},order=1000)
    public void defaultClearingN1000(BaseTransactionVo bo) {
        Crdgrp crdgrp = Utils.getFieldValue(bo, "crdgrp");
        if (Objects.isNull(crdgrp)) {
            return;
        }
        // 给汇入清算/汇出清算赋值的清算类型赋值
        if (MdaUtils.isEmpty(crdgrp.getRec().getClrtyp())) {
            if (sysmodService.sptIsRimLoaded(bo.getSpt().getSta())) {
                // 从Register队列进的全部是汇出清算
                crdgrp.getRec().setClrtyp("O");
                // 自动清算
                crdgrp.getRec().setTrntyp("A");
            } else {
                if (sysmodService.sptIsIncLoaded(bo.getSpt().getSta())) {
                    // 从Incoming队列进的全部是汇入清算
                    crdgrp.getRec().setClrtyp("I");
                    // 自动清算
                    crdgrp.getRec().setTrntyp("A");
                } else {
                    // 其他情况，根据交易名称判断
                    if (MdaUtils.compareTo(SettleContext.getTransName(), "CRTCRO") == 0) {
                        crdgrp.getRec().setClrtyp("O");
                    } else {
                        crdgrp.getRec().setClrtyp("I");
                    }
                    // 手工清算
                    crdgrp.getRec().setTrntyp("M");
                }
            }
        }


    }


    /**
     * source:crtp.0006.script
     */
    public List<Map<String, Object>> setBranchCodeValue() {
        //########################
        //!初始化实体机构
        //########################
        String ety = SettleContext.getUserEty().getExtkey();
        String sql = "SELECT INR as val , BRANCH||BCHNAME as txt FROM BCH WHERE bchtyp = '1' AND ETYEXTKEY='" + ety + "'";
//        String sql = "SELECT INR as val , BRANCH||BCHNAME as txt FROM BCH WHERE wrkflg = 'X' AND ETYEXTKEY='" + ety + "'";
        List<Map<String, Object>> list = publicMapper.dyncQueryMapList(ImmutableBiMap.of("sql", sql));
        return list;
    }

}
