package com.brilliance.isc.funds.crtcro.service;

import cn.hutool.core.bean.BeanUtil;
import com.brilliance.isc.bo.*;
import com.brilliance.isc.bo.model.*;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.component.PtsptaToolComponent;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.exception.CommonServiceException;
import com.brilliance.isc.common.nosmod.NosmodService;
import com.brilliance.isc.common.setmod.vo.SetmodVo;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.funds.bo.funds.CrtcroStoreBo;
import com.brilliance.isc.funds.common.component.CrToolComponent;
import com.brilliance.isc.funds.crtcro.check.CrtcroValidator;
import com.brilliance.isc.mda.dao.ActMapper;
import com.brilliance.isc.mda.dao.GleMapper;
import com.brilliance.isc.mda.dao.PtyMapper;
import com.brilliance.isc.mda.dao.SmhMapper;
import com.brilliance.isc.vo.funds.CrdBizInfoVo;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.google.common.collect.ImmutableBiMap;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: CrtcroTransactionServiceImpl
 * @description: 汇出清算 Crtcro 业务处理
 * @author: huangshunlin
 * @create: 2024-10-17 22:42
 */
@Service("crtcro.transaction")
public class CrtcroTransactionServiceImpl extends AbstractTransactionService<CrtcroStoreBo> {

    @Resource
    private CrtcroValidator crtcroValidator;

    @Resource
    private SysmodService sysmodService;

    @Resource
    private CrToolComponent crToolComponent;

    @Resource
    private ActMapper actMapper;

    @Resource
    private PtsptaToolComponent ptsptaToolComponent;

    @Resource
    private TransactionHelpService transactionHelpService;

    @Resource
    private SettleContext settleContext;

    @Resource
    private CbsmodService cbsmodService;

    @Resource
    private TrnmodService trnmodService;

    @Autowired
    private PtaCacheService ptaCacheService;

    @Resource
    private BchService bchService;

    @Autowired
    private NosmodService nosmodService;


    @Resource
    private SmhMapper smhMapper;

    @Resource
    private GleMapper gleMapper;

    @Resource
    private PtyMapper ptyMapper;

    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "N");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_JSH_SERVICE, "Y");
    }


    @Override
    protected void txnSave(CrtcroStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    /**
     * source:crtcro.0020.script
     */
    public void ruleMtabutSavN500(CrtcroStoreBo bo) {
        Crdgrp crdgrp = bo.getCrdgrp();
        crToolComponent.getref(crdgrp);
        // 1. Create (rsp. update) Contract (incl. Texts)
        // change state
        bo.getCrdgrp().getRec().setSta("D");
        crToolComponent.sav(crdgrp);
        // 2. Build Transaction from CRDGRP
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();
        trnmodService.trnSetFromContract(trnmodVo);
        cbsSav(bo);
        SetmodVo setmodVo = transactionHelpService.getTransactionStoreSet().getSetmodVo();
        nosmodService.savNostro(bo.getNosmod(), setmodVo, trnmodVo);

    }

    /**
     * source:crtcro.0021.script
     */
    private void cbsSav(CrtcroStoreBo bo) {
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();
        String trninr = transactionHelpService.getTrninr();
        cbsmodService.cbsTrnSetNew("TRN", trninr);
        trnmodService.cbsTrnStore("MAXAMT", "AMT1", Dates.today(), bo.getCrdgrp().getCbs().getMax().getCur(), bo.getCrdgrp().getCbs().getMax().getAmt()
                , I18nUtil.getMessage("funds_crtcr0_000000"), bo.getCrdgrp().getRec(), trninr);
    }

    @Override
    protected Map<String, String> txnCheck(CrtcroStoreBo transactionVo) {
        return crtcroValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(CrtcroStoreBo transactionVo) {
        Crd rec = transactionVo.getCrdgrp().getRec();
        Crtp crtp = transactionVo.getCrtp();
        CrdBizInfoVo bizInfoVo = new CrdBizInfoVo();
        bizInfoVo.setObjinr(rec.getInr());
        bizInfoVo.setObjtyp("CRD");
        bizInfoVo.setForare(crtp.getForare());
        bizInfoVo.setOpndat(rec.getOpndat());
        bizInfoVo.setRcvbchinr(rec.getRcvbchinr());
        bizInfoVo.setCustyp(rec.getCustyp());
        String act = transactionVo.getSpcgle().getAct();
        bizInfoVo.setSpcgleAct(act);
        bizInfoVo.setRcvobjtyp(rec.getRcvobjtyp());
        Cpd cpd = transactionVo.getCpdgrp().getRec();
        bizInfoVo.setValdat(cpd.getValdat());
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(CrtcroStoreBo transactionVo) {
        return null;
    }

    @Override
    public CrtcroStoreBo init(CrtcroStoreBo crtcroStoreBo) {
        crtcroStoreBo.reset();
        init950(crtcroStoreBo);
        init1000(crtcroStoreBo);
        return crtcroStoreBo;
    }

    /**
     * source:crtcro.0001.script
     */
    public void init1000(CrtcroStoreBo bo) {
        // 不让Break
//        Systems.resetVisible(bo.getMtabut(), "pen");
//        Systems.setValues(bo.getCrdgrp().getRec(), "rcvobjtyp", "CPD" + CR + "BCD" + CR + "BRD" + CR + "GID");
        String msgTyp = NULLSTR;
        String ptyInr = NULLSTR;
        if (sysmodService.sptIsRimLoaded(bo.getSpt().getSta())) {
            // 汇出清算通过Register队列进入
            bo.getCrdgrp().getRec().setClrtyp("O");
            bo.getCrdgrp().getRec().setCredat(Dates.today());
            bo.getCrdgrp().getRec().setOpndat(Dates.today());
            bo.getCrdgrp().getRec().setSptinr(bo.getSpt().getInr());
            // 查找TRN记录
            bo.getCretrn().setInr(bo.getSpt().getCreobjinr());
            Trn dbTrn = (Trn) sysmodService.sdbRead(bo.getCretrn());
            if (Objects.isNull(dbTrn)) {
                throw new CommonServiceException("根据Creobjinr:" + bo.getSpt().getCreobjinr() + "找不到TRN数据");
            }
            bo.setCretrn(dbTrn);
            bo.getCrdgrp().getRec().setRcvobjtyp(bo.getCretrn().getObjtyp());
            bo.getCrdgrp().getRec().setRcvobjinr(bo.getCretrn().getObjinr());
            bo.getCrdgrp().getRec().setRcvbchinr(bo.getCretrn().getBchkeyinr());
            bo.getCrdgrp().getRec().setMsgref(bo.getCretrn().getOwnref());
            // 查找SMH记录（202）
            bo.getBndsmh().setInr(bo.getSpt().getSmhinr());
            Smh dbSmh = (Smh) sysmodService.sdbRead(bo.getBndsmh());
            if (Objects.isNull(dbSmh)) {
                throw new CommonServiceException("根据Smhinr:" + bo.getSpt().getSmhinr() + "找不到SMH数据");
            }
            bo.setBndsmh(dbSmh);
//            ctx.getDaoSession().dbRead(bo.getBndsmh(), bo.getSpt().getSmhinr(), "INR");
            bo.getCrdgrp().getRec().setBndinr(bo.getBndsmh().getInr());
            bo.getCrdgrp().getRec().setBndtyp(bo.getBndsmh().getMsgtyp());
            // 查找绑定这笔202的其他SWIFT报文信息
            switch (bo.getCrdgrp().getRec().getRcvobjtyp()) {
                case "CPD":
//                    Platform.getCPdGrp(this.getCpdgrp(),this.getCrdgrp().getRec().getRcvobjinr());
                    Cpdgrp dbCpdgrp = crToolComponent.getCpdgrpByInr(bo.getCrdgrp().getRec().getRcvobjinr());
                    bo.setCpdgrp(dbCpdgrp);
                    // 汇出汇款发103+202
                    if (MdaUtils.compareTo(bo.getCpdgrp().getRec().getSwftyp(), "COV") == 0) {
                        msgTyp = "103";
                    }
                    break;
                case "BCD":
                    msgTyp = "400";
//                    Platform.getBcdGrp(bo.getBcdgrp(), bo.getCrdgrp().getRec().getRcvobjinr());
                    Bcdgrp dbBcdgrp = crToolComponent.getBcdgrpByInr(bo.getCrdgrp().getRec().getRcvobjinr());
                    bo.setBcdgrp(dbBcdgrp);
                    break;
                case "BRD":
                    msgTyp = "756";
//                    Platform.getBrdGrp(bo.getBrdgrp(), bo.getCrdgrp().getRec().getRcvobjinr());
//                    Platform.getLidGrp(bo.getLidgrp(), bo.getBrdgrp().getRec().getPntinr());
                    Brdgrp dbBrdgrp = crToolComponent.getBrdgrpByInr(bo.getCrdgrp().getRec().getRcvobjinr());
                    bo.setBrdgrp(dbBrdgrp);
                    Lidgrp dbLidgrp = crToolComponent.getLidgrpByInr(bo.getBrdgrp().getRec().getPntinr());
                    bo.setLidgrp(dbLidgrp);
                    break;
                case "GID":
                    msgTyp = "";
//                    Platform.getGidGrp(bo.getGidgrp(), bo.getCrdgrp().getRec().getRcvobjinr());
                    Gidgrp dbGidgrp = crToolComponent.getGidgrpByInr(bo.getCrdgrp().getRec().getRcvobjinr());
                    bo.setGidgrp(dbGidgrp);
                    break;
                default:
                    msgTyp = "";
            }
            if (!MdaUtils.isEmpty(msgTyp)) {
//                ctx.getDaoSession().dbRead(bo.getCresmh(), "WHERE TRNTYP = 'TRN' AND TRNINR = '" + bo.getBndsmh().getTrninr() + "' AND MSGTYP = '" + msgTyp + "'");
                List<Smh> smhList = smhMapper.selectByTrninrAndMsgtyp(bo.getBndsmh().getTrninr(), msgTyp);
                if (CollectionUtils.isEmpty(smhList)) {
                    // 找不到，可能发了999
//                    ctx.getDaoSession().dbRead(bo.getCresmh(), "WHERE TRNTYP = 'TRN' AND TRNINR = '" + bo.getBndsmh().getTrninr() + "' AND MSGTYP = '999'");
                    List<Smh> smhList2 = smhMapper.selectByTrninrAndMsgtyp(bo.getBndsmh().getTrninr(), "999");
                    if (CollectionUtils.isNotEmpty(smhList2)) {
                        bo.setCresmh(smhList2.get(0));
                    }
                } else {
                    bo.setCresmh(smhList.get(0));
                }
                bo.getCrdgrp().getRec().setMsginr(bo.getCresmh().getInr());
                bo.getCrdgrp().getRec().setMsgtyp(bo.getCresmh().getMsgtyp());
            } else {
                //    CRTP\\ACCP\\PANSTA = PanStaShow
//                Platform.setModuleContent(bo.getCresmh(), bo.getBndsmh());
                Smh smh = MdaUtils.deepClone(bo.getBndsmh());
                bo.setCresmh(smh);
//                MdaUtils.clear(bo.getBndsmh());
                bo.setBndsmh(new Smh());
                bo.getCrdgrp().getRec().setBndinr(NULLSTR);
                bo.getCrdgrp().getRec().setBndtyp(NULLSTR);
                bo.getCrdgrp().getRec().setMsginr(bo.getCresmh().getInr());
                bo.getCrdgrp().getRec().setMsgtyp(bo.getCresmh().getMsgtyp());
            }
            // 读取前手交易的清算汇出账务信息
//            ctx.getDaoSession().dbRead(bo.getSpcgle(), "WHERE TRNINR = '" + bo.getCretrn().getInr() + "' AND DBTCDT = 'C' AND ( DBTDFT ='SP-ROC' OR DBTDFT = 'SP-ROP') ");
            Gle gle = gleMapper.selectSpcGLeByTrninr(bo.getCretrn().getInr());
            bo.setSpcgle(gle);
            if (MdaUtils.compareTo(bo.getSpcgle().getDbtdft(), "SP-ROP") == 0) {
                bo.getCrdgrp().getRec().setCustyp("1");
            } else {
                bo.getCrdgrp().getRec().setCustyp("0");
            }
            bo.getCrdgrp().getCbs().getMax().setCur(bo.getSpcgle().getCur());
            bo.getCrdgrp().getCbs().getMax().setAmt(bo.getSpcgle().getAmt());
            // 将收报行信息赋值到PTSPTA中
            if (!MdaUtils.isEmpty(bo.getCresmh().getSndkey())) {
                // 说明有主报文（103/400/756）
                Pty crePty = ptyMapper.selectByExtkey(bo.getCresmh().getSndkey());
                ptsptaToolComponent.ptsmodGetPtsptaFromPtyINR(bo.getCrdgrp().getRcv(), crePty.getInr());
                if (!MdaUtils.isEmpty(bo.getBndsmh().getSndkey())) {
                    // 副报文（202）读取到账户行中
                    Pty bndPty = ptyMapper.selectByExtkey(bo.getBndsmh().getSndkey());
                    ptsptaToolComponent.ptsmodGetPtsptaFromPtyINR(bo.getCrdgrp().getAcc(), bndPty.getInr());
                }
            }
            // 读取同业账户116信息
            if (!MdaUtils.isEmpty(bo.getCrdgrp().getAcc().getPts().getPtyinr())) {
                ptyInr = bo.getCrdgrp().getAcc().getPts().getPtyinr();
            } else {
                ptyInr = bo.getCrdgrp().getRcv().getPts().getPtyinr();
            }
//            readAct(bo.getSysmod().getBch().getPtyinr(), ptyInr, bo.getPacact());
            String maxCur = bo.getCrdgrp().getCbs().getMax().getCur();

            readAct("", ptyInr, bo.getPacact(), maxCur);
//            if (Objects.isNull(bo.getPacact()) || StringUtils.isEmpty(bo.getPacact().getInr())) {
//                MdaUtils.clear(bo.getPacact());
//            }
        }


    }

    private void init950(CrtcroStoreBo crtcroStoreBo) {
        crtcroStoreBo.setClrsta("O");
        String extkey = SettleContext.getUserSession().getUsr().getExtkey();
        crtcroStoreBo.getCrdgrp().getRec().setOwnusr(extkey);
    }

    @Override
    public void beforeSave(CrtcroStoreBo transactionVo) {
        transactionVo.reset();
        transactionVo.reset();
        Crdgrp crdgrp = transactionVo.getCrdgrp();
        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(crdgrp.getRec());
        transactionServiceSet.setTransactionVo(transactionVo);
        crToolComponent.collectCbsMap(transactionServiceSet, "", crdgrp.getCbs(), "max");
        crToolComponent.collectPtsList(transactionServiceSet, crdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());

    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, CrtcroStoreBo transactionVo) {
        bizInfoVo.setObjinr(transactionVo.getCrdgrp().getRec().getInr());

    }

    public String getRef(CrtcroStoreBo bo) {
        bo.reset();
        if (Objects.isNull(SettleContext.getUserSession().getWrkbch()) || StringUtils.isEmpty(SettleContext.getUserSession().getWrkbch().getInr())) {
            SettleContext.getUserSession().setWrkbch(SettleContext.getUserSession().getBch());
        }
        Crdgrp crdgrp = bo.getCrdgrp();
        crToolComponent.getref(crdgrp);
        return crdgrp.getRec().getOwnref();
    }

    /**
     * source:crtcro.0006.script
     *
     * @param holPtyInr
     * @param serPtyInr
     * @param act
     * @param cur       this.getCrdgrp().getCbs().getMax().getCur()
     */
    public void readAct(String holPtyInr, String serPtyInr, Act act, String cur) {
        String sql = "SELECT * from act WHERE HOLPTYINR = '" + holPtyInr + "' AND SERPTYINR = '" + serPtyInr + "' AND CUR = '" + cur + "' AND TYP = 'DB'";
        List<Act> actList = actMapper.dyncRead(ImmutableBiMap.of("sql", sql));
        if (CollectionUtils.isNotEmpty(actList)) {
            BeanUtil.copyProperties(actList.get(0), act);
        }
    }

}
