package com.brilliance.isc.funds.fttfcm.service;

import com.brilliance.isc.bo.Act;
import com.brilliance.isc.bo.Bch;
import com.brilliance.isc.bo.Ftd;
import com.brilliance.isc.bo.Pta;
import com.brilliance.isc.bo.model.Ftdgrp;
import com.brilliance.isc.bo.model.Fttp;
import com.brilliance.isc.funds.transfer.transaction.FttfcmStructMapper;
import com.brilliance.isc.funds.vo.funds.FttfcmStoreVo;
import com.brilliance.isc.vo.model.FtdgrpVo;
import com.brilliance.isc.vo.model.FttpVo;
import com.brilliance.isc.vo.model.FtdVo;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.cbsmod.service.XrtmodService;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.liaall.service.LiaallService;
import com.brilliance.isc.common.setglg.vo.SetglgVo;
import com.brilliance.isc.common.setmod.service.SetmodService;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.funds.bo.funds.FttfcmStoreBo;
import com.brilliance.isc.funds.common.component.FtToolComponent;
import com.brilliance.isc.funds.fttfcm.check.FttfcmValidator;
import com.brilliance.isc.mda.dao.PtyratMapper;
import com.brilliance.isc.vo.funds.FtdBizInfoVo;
import com.brilliance.mda.runtime.mda.util.*;
import com.fasterxml.jackson.annotation.JsonIgnore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.brilliance.isc.mda.dao.ActMapper;
import com.brilliance.isc.common.component.SptDataComponent;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static com.brilliance.mda.runtime.mda.Constants.*;
import static com.brilliance.mda.runtime.mda.Constants.CR;

/**
 * @program: isc-funds
 * @ClassName: FttfcmTransactionServiceImpl
 * @description: 结售汇平盘确认 Fttfcm 业务处理
 * @author: xiaotong
 * @create: 2024-10-11 15:20
 */

@Service("fttfcm.transaction")
public class FttfcmTransactionServiceImpl extends AbstractTransactionService<FttfcmStoreBo> {
    @Resource
    private FttfcmValidator fttfcmValidator;
//    @Autowired
//    private FttfcmService fttfcmService;
    @Resource
    private FtToolComponent ftToolComponent;
    @Resource
    private TrnmodService trnmodService;
    @Resource
    private XrtmodService xrtmodService;
    @Resource
    private TransactionHelpService transactionHelpService;
    @Resource
    private CbsmodService cbsmodService;
    @Resource
    private LiaallService liaallService;
    @Resource
    private SettleContext settleContext;
    @Autowired
    private PtaCacheService ptaCacheService;

    @Resource
    private BchService bchService;

    @Resource
    private SysmodService sysmodService;

    @Resource
    private PtyratMapper ptyratMapper;

    @Resource
    private ActMapper actMapper;

    @Resource
    private SetmodService setmodService;

    @Autowired
    private FttfcmStructMapper fttfcmStructMapper;


    @Autowired
    SptDataComponent sptDataComponent;

    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDIA_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");

//        serviceRegisterMaps.put(CommonContants.UMDMOD_SERVICE, "N");
//        serviceRegisterMaps.put(CommonContants.LIMMOD_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.BOPSET_RMB_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.BOPSET_SZSB_BOPREM_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.BOPSET_CFABOP_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.FANXIQIAN_SERVICE, "Y");
    }

    @Override
    protected void txnSave(FttfcmStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    @Override
    protected Map<String, String> txnCheck(FttfcmStoreBo transactionVo) {
        return fttfcmValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FttfcmStoreBo transactionVo) {
        FtdBizInfoVo bizInfoVo = new FtdBizInfoVo();
        bizInfoVo.setObjtyp("FTD");
        Ftd rec = transactionVo.getFtdgrp().getRec();
        bizInfoVo.setObjinr(rec.getInr());
        bizInfoVo.setOwnref(rec.getOwnref());
        bizInfoVo.setFttyp(rec.getFttyp());
        bizInfoVo.setRat(rec.getRat());
        bizInfoVo.setValdat(rec.getValdat());
        bizInfoVo.setBnktyp(rec.getBnktyp());
        bizInfoVo.setMsgtyp(transactionVo.getFttp().getMsgtyp());
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FttfcmStoreBo transactionVo) {
        return null;
    }

    @Override
    public FttfcmStoreBo init(FttfcmStoreBo fttfcmStoreBo) {
        fttfcmStoreBo.reset();

        init950(fttfcmStoreBo);
        init1000(fttfcmStoreBo);
        initFtdgrpRecOpndatN1000(fttfcmStoreBo);

        return fttfcmStoreBo;
    }

    private void init950(FttfcmStoreBo fttfcmStoreBo) {
    }

    private void init1000(FttfcmStoreBo bo) {
        if( ! sysmodService.sptIsAnyLoaded(bo.getSpt().getSta()) )
        {
            bo.getFttp().setMsgtyp( "320");
        }

        String extkey = SettleContext.getUserSession().getUsr().getExtkey();
        bo.getFtdgrp().getRec().setOwnusr(extkey);
        bo.getFtdgrp().getRec().setUsr(extkey);
        bo.getFtdgrp().getRec().setBnktyp("9");
        bo.getFttp().setBnaktypflg("X");


    }

    @Override
    public void beforeSave(FttfcmStoreBo transactionVo) {
        transactionVo.reset();
        Ftdgrp ftdgrp = transactionVo.getFtdgrp();

        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(ftdgrp.getRec());
        transactionServiceSet.setBlk(ftdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        ftToolComponent.collectCbsMap(transactionServiceSet, "", ftdgrp.getCbs(), "max", "ins", "opn1", "opn2");
        ftToolComponent.collectPtsList(transactionServiceSet, ftdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FttfcmStoreBo transactionVo) {
        bizInfoVo.setObjinr(transactionVo.getFtdgrp().getRec().getInr());
    }

    public void ruleMtabutSavN500(FttfcmStoreBo bo) {
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();

        ftToolComponent.getref(bo.getFtdgrp());
        // 1. Create (rsp. update) Contract (incl. Texts)
        bo.getFtdgrp().getRec().setCnfdat( Dates.today());

        ftToolComponent.sav(bo.getFtdgrp());
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract ( trnmodVo);
        cbsSav(bo,trnmodVo);
        // save documents
        //TODO
        //记录116台账
        //transactionVo.getNosmod().savNostro();
    }

    public String getRef(FttfcmStoreVo fttfcmStorevo) {
        FttfcmStoreBo fttfcmStoreBo= fttfcmStructMapper.voTransferToBo(fttfcmStorevo);
        return getRef(fttfcmStoreBo);
    }

    public String getRef(FttfcmStoreBo fttfcmStoreBo) {
        fttfcmStoreBo.reset();
        PtsptaVo ptsptaVo = fttfcmStoreBo.getFtdgrp().getTro();

        if (ptsptaVo == null || StringUtils.isEmpty(ptsptaVo.getPtainr())) {
            SettleContext.getUserSession().setWrkbch(SettleContext.getUserSession().getBch());
        }else{
            Pta pta = ptaCacheService.getPtaByInr(ptsptaVo.getPtainr());
            Bch wrkbch = new Bch();
            wrkbch.setInr(pta.getBranchinr());
            wrkbch = bchService.selectByPrimaryKey(wrkbch);
            SettleContext.getUserSession().setWrkbch(wrkbch);
        }

        Ftdgrp ftdgrp = fttfcmStoreBo.getFtdgrp();
        ftToolComponent.getref(ftdgrp);
        return ftdgrp.getRec().getOwnref();
    }

    public void cbsSav(FttfcmStoreBo transactionVo, TrnmodVo trnmodVo) {
        String trninr = transactionHelpService.getTrninr();


        cbsmodService.cbsTrnSetNew("TRN",trnmodVo.getTrn().getInr());

        trnmodService.cbsTrnStore("MAXAMT","AMT1",Dates.today(),transactionVo.getFtdgrp().getCbs().getMax().getCur(),transactionVo.getFtdgrp().getCbs().getMax().getAmt(),MdaUtils.getI18NString("fttfcm","CT000003"),transactionVo.getFtdgrp().getRec(),trninr);
        trnmodService.cbsTrnStore("INSAMT","AMT1",Dates.today(),transactionVo.getFtdgrp().getCbs().getIns().getCur(),transactionVo.getFtdgrp().getCbs().getIns().getAmt(),MdaUtils.getI18NString("fttfcm","CT000009"),transactionVo.getFtdgrp().getRec(),trninr);
        if( MdaUtils.compareTo(transactionVo.getFtdgrp().getRec().getFttyp(),"LI") == 0 || MdaUtils.compareTo(transactionVo.getFtdgrp().getRec().getFttyp(),"DI") == 0 )
        {
            trnmodService.cbsTrnStore("OPNAMT","AMT1",Dates.today(),transactionVo.getFtdgrp().getCbs().getMax().getCur(),transactionVo.getFtdgrp().getCbs().getMax().getAmt(),MdaUtils.getI18NString("fttfcm","CT000010"),transactionVo.getFtdgrp().getRec(),trninr);
        }
        cbsmodService.cbsTrnCommit();




    }


    //Event fttp.msgtyp
    public boolean ruleFttpMsgtypN1000(FttfcmStoreBo vo){
        vo.reset();

        SetglgVo setglgVo=vo.getSetglg();

        for(int  i = 1;i <= setglgVo.getSetgll().size();i++)
        {
            setglgVo.getSetgll().get(i).setMacold(NULLSTR);
        }


        return true;
    }




    /**
     *   source:fttfcm.0031.script
     */
    @JsonIgnore
    public void setMTn99(FttfcmStoreBo fttfcmStoreBo)
    {
        Ftdgrp ftdgrp=fttfcmStoreBo.getFtdgrp();
        Fttp fttp=fttfcmStoreBo.getFttp();

        String blk = "ATTN:NANFANG SUB-BRANCH";
        blk = blk + CR + "PLS DEBIT OUR A/C NO. " + fttp.getTriact() + " WITH " + Formats.fmtAmount(ftdgrp.getCbs().getMax().getAmt(),ftdgrp.getCbs().getMax().getCur()) + " FOR FIXED DEPOSIT DEAL, DETAILS BELOW:";
        blk = blk + CR + "VALUE DATE: " + Formats.format(ftdgrp.getRec().getValdat(),"YYYYMMDD");
        blk = blk + CR + "MATURITY DATE: " + Formats.format(ftdgrp.getRec().getMatdat(),"YYYYMMDD");
        blk = blk + CR + "CURRENCY AND AMOUNT: " + ftdgrp.getCbs().getMax().getCur() + Formats.fmtAmount(ftdgrp.getCbs().getMax().getAmt(),ftdgrp.getCbs().getMax().getCur());
        blk = blk + CR + "INTEREST DUE DATE: " + Formats.format(ftdgrp.getRec().getMatdat(),"YYYYMMDD");
        blk = blk + CR + "CURRENCY AND INTEREST AMT: " + ftdgrp.getCbs().getIns().getCur() + Formats.fmtAmount(ftdgrp.getCbs().getIns().getAmt(),ftdgrp.getCbs().getIns().getCur());
        blk = blk + CR + "INTEREST RATE: " + MdaUtils.toString(ftdgrp.getRec().getRat());
        blk = blk + CR + "DATCOUNT FRACTION: " + ftdgrp.getRec().getCntfra();
        blk = blk + CR + "PLS EFFECT OVER OUR ACCOUNT.";
        blk = blk + CR + ".";
        blk = blk + CR + "BEST REGARD";
        blk = blk + CR + "IBD";
        fttfcmStoreBo.setNarblk( blk);
        if( MdaUtils.compareTo(ftdgrp.getRec().getFttyp(),"DO") == 0 || MdaUtils.compareTo(ftdgrp.getRec().getFttyp(),"LO") == 0 )
        {
            fttfcmStoreBo.setRelref( ftdgrp.getTri().getPts().getRef());
        }
        else
        {
            fttfcmStoreBo.setRelref( ftdgrp.getTro().getPts().getRef());
        }

    }



    
    public boolean defaultSwtn99N1000(FttfcmStoreBo fttfcmStoreBo){
        Ftdgrp ftdgrp=fttfcmStoreBo.getFtdgrp();
        Fttp fttp=fttfcmStoreBo.getFttp();
        
        String blk = NULLSTR;
        if(!fttfcmStoreBo.getFtdgrp().getRec().isModified("narblk"))
        {
            blk = "ATTN:NANFANG SUB-BRANCH";
            blk = blk + CR + "PLS DEBIT OUR A/C NO. " + fttp.getTriact() + " WITH " + Formats.fmtAmount(ftdgrp.getCbs().getMax().getAmt(),ftdgrp.getCbs().getMax().getCur()) + " FOR FIXED DEPOSIT DEAL, DETAILS BELOW:";
            blk = blk + CR + "VALUE DATE: " + Formats.format(ftdgrp.getRec().getValdat(),"YYYYMMDD");
            blk = blk + CR + "MATURITY DATE: " + Formats.format(ftdgrp.getRec().getMatdat(),"YYYYMMDD");
            blk = blk + CR + "CURRENCY AND AMOUNT: " + ftdgrp.getCbs().getMax().getCur() + Formats.fmtAmount(ftdgrp.getCbs().getMax().getAmt(),ftdgrp.getCbs().getMax().getCur());
            blk = blk + CR + "INTEREST DUE DATE: " + Formats.format(ftdgrp.getRec().getMatdat(),"YYYYMMDD");
            blk = blk + CR + "CURRENCY AND INTEREST AMT: " + ftdgrp.getCbs().getIns().getCur() + Formats.fmtAmount(ftdgrp.getCbs().getIns().getAmt(),ftdgrp.getCbs().getIns().getCur());
            blk = blk + CR + "INTEREST RATE: " + MdaUtils.toString(ftdgrp.getRec().getRat());
            blk = blk + CR + "DATCOUNT FRACTION: " + ftdgrp.getRec().getCntfra();
            blk = blk + CR + "PLS EFFECT OVER OUR ACCOUNT.";
            blk = blk + CR + ".";
            blk = blk + CR + "BEST REGARD";
            blk = blk + CR + "IBD";
            fttfcmStoreBo.setNarblk( blk);
        }
        if(!fttfcmStoreBo.getFtdgrp().getRec().isModified("relref"))
        {
            if( MdaUtils.compareTo(ftdgrp.getRec().getFttyp(),"DO") == 0 || MdaUtils.compareTo(ftdgrp.getRec().getFttyp(),"LO") == 0 )
            {
                fttfcmStoreBo.setRelref( ftdgrp.getTri().getPts().getRef());
            }
            else
            {
                fttfcmStoreBo.setRelref( ftdgrp.getTro().getPts().getRef());
            }
        }


        return true;
    }


    /**
     *   source:fttfcm.0035.script
     */
    //@Default(target="fttp.triact",order=1200)
    public List<String> defaultFttpTriactN1200(FttfcmStoreBo fttfcmStoreBo){
        Ftdgrp ftdgrp=fttfcmStoreBo.getFtdgrp();
        Fttp fttp=fttfcmStoreBo.getFttp();

        /**
         if not IsEmpty( FTDGRP\\CBS\\MAX\\CUR ) and not IsEmpty( FTDGRP\\ACT\\PTS\\PTYINR ) and ( FTDGRP\\REC\\FTTYP == "DO" or FTDGRP\\REC\\FTTYP == "LO" ) then

         **/
        List<String> actLst=new ArrayList<>();
        if( ! MdaUtils.isEmpty(ftdgrp.getCbs().getMax().getCur()) && ! MdaUtils.isEmpty(ftdgrp.getAct().getPts().getPtyinr()) )
        {
            String sql = "WHERE CUR = '" + ftdgrp.getCbs().getMax().getCur() + "' AND SERPTYINR = '" + ftdgrp.getTri().getPtyinr() + "' AND TYP = 'DB'";
            List<String > actList = actMapper.getSerAccByConditionSql(sql);
            if( ! fttp.isModified("triact") && actLst.size()>0) {
                fttp.setTriact( actLst.get(1));
            }
        }
        else{
            fttp.setTriact(NULLSTR);
        }

        return actLst;
    }



    public List<String> defaultFttpTroactN1200(FttfcmStoreBo fttfcmStoreBo){
        Ftdgrp ftdgrp=fttfcmStoreBo.getFtdgrp();
        Fttp fttp=fttfcmStoreBo.getFttp();

        /**
         if not IsEmpty( FTDGRP\\CBS\\MAX\\CUR ) and not IsEmpty( FTDGRP\\ACT\\PTS\\PTYINR ) and ( FTDGRP\\REC\\FTTYP == "DO" or FTDGRP\\REC\\FTTYP == "LO" ) then

         **/
        List<String> actLst=new ArrayList<>();
        if( ! MdaUtils.isEmpty(ftdgrp.getCbs().getMax().getCur()) && ! MdaUtils.isEmpty(ftdgrp.getAct().getPts().getPtyinr()) )
        {
            String sql = "WHERE CUR = '" + ftdgrp.getCbs().getMax().getCur() + "' AND SERPTYINR = '" + ftdgrp.getTro().getPtyinr() + "' AND TYP = 'DB'";
            List<String > actList = actMapper.getSerAccByConditionSql(sql);
            if( ! fttp.isModified("troact") && actLst.size()>0) {
                fttp.setTroact( actLst.get(1));
            }
        }
        else{
            fttp.setTroact(NULLSTR);
        }

        return actLst;
    }

    /**
     *   source:fttfcm.0003.script
     */
    //@Default(target="ftdgrp.rec.opndat",order=1000)
    public boolean initFtdgrpRecOpndatN1000(FttfcmStoreBo fttfcmStoreBo){


        if( MdaUtils.isEmpty(fttfcmStoreBo.getFtdgrp().getRec().getOpndat()) )
        {
            fttfcmStoreBo.getFtdgrp().getRec().setOpndat(  Dates.today());
        }

        if( MdaUtils.isEmpty(fttfcmStoreBo.getFtdgrp().getRec().getValdat()) )
        {
            fttfcmStoreBo.getFtdgrp().getRec().setValdat(  Dates.today());
        }

        return true;
    }


    /**
     *   source:fttfcm.0019.script
     */
    //@Default(target="ftdgrp.cbs.ins.amt",order=1000)
    public boolean defaultFtdgrpCbsInsAmtN1000(FttfcmStoreBo fttfcmStoreBo){

        switch( fttfcmStoreBo.getFtdgrp().getRec().getCntfra() )
        {
            case "ACT/365":
            case  "AFI/365":
                fttfcmStoreBo.getFtdgrp().getCbs().getIns().setAmt( xrtmodService.amtRnd( Decimals.div(Decimals.mul(Decimals.div(Decimals.mul(fttfcmStoreBo.getFtdgrp().getCbs().getMax().getAmt(),fttfcmStoreBo.getFtdgrp().getRec().getRat()),100),Dates.diff(fttfcmStoreBo.getFtdgrp().getRec().getMatdat(),fttfcmStoreBo.getFtdgrp().getRec().getValdat())),365), fttfcmStoreBo.getFtdgrp().getCbs().getIns().getCur() ));
                break;
            case "ACT/360":
                fttfcmStoreBo.getFtdgrp().getCbs().getIns().setAmt( xrtmodService.amtRnd( Decimals.div(Decimals.mul(Decimals.div(Decimals.mul(fttfcmStoreBo.getFtdgrp().getCbs().getMax().getAmt(),fttfcmStoreBo.getFtdgrp().getRec().getRat()),100),Dates.diff(fttfcmStoreBo.getFtdgrp().getRec().getMatdat(),fttfcmStoreBo.getFtdgrp().getRec().getValdat())),360), fttfcmStoreBo.getFtdgrp().getCbs().getIns().getCur() ));
                break;
            default:
                fttfcmStoreBo.getFtdgrp().getCbs().getIns().setAmt( new BigDecimal("0"));
        }
        return true;
    }


    /**
     *   source:fttfcm.0020.script
     */
    //@Default(target="ftdgrp.cbs.ins.cur",order=1000)
    public boolean defaultFtdgrpCbsInsCurN1000(FttfcmStoreBo fttfcmStoreBo){
        fttfcmStoreBo.getFtdgrp().getCbs().getIns().setCur(  fttfcmStoreBo.getFtdgrp().getCbs().getMax().getCur());

        return true;
    }

}
