package com.brilliance.isc.funds.fxtlop.service.impl;

import com.brilliance.isc.bo.*;
import com.brilliance.isc.bo.model.Fxacbs;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.cbsmod.service.XrtmodService;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.liaall.service.LiaallService;
import com.brilliance.isc.common.setmod.service.SetmodService;
import com.brilliance.isc.common.setmod.vo.SetmodVo;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.funds.bo.funds.FxtlopStoreBo;
import com.brilliance.isc.funds.bo.funds.FxtssbStoreBo;
import com.brilliance.isc.funds.common.component.FxToolComponent;
import com.brilliance.isc.funds.fxtlop.check.FxtlopValidator;
import com.brilliance.isc.funds.fxtlop.service.FxtlopService;
import com.brilliance.isc.funds.fxtlop.service.FxtpService;
import com.brilliance.isc.funds.vo.funds.FxtlopStoreVo;
import com.brilliance.isc.funds.vo.funds.FxtssbStoreVo;
import com.brilliance.isc.mda.dao.PtyratMapper;
import com.brilliance.isc.vo.funds.FxdBizInfoVo;
import com.brilliance.mda.runtime.mda.impl.Argument;
import com.brilliance.mda.runtime.mda.util.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: FxtlopTransactionServiceImpl
 * @description: 结售汇平盘登记 Fxtlop 业务处理
 * @author: wangweidong
 * @create: 2024-10-08 16:35
 */
@Service("fxtlop.transaction")
public class FxtlopTransactionServiceImpl extends AbstractTransactionService<FxtlopStoreBo> {

    @Resource
    private FxtlopValidator fxtlopValidator;
    @Autowired
    private FxtlopService fxtlopService;
    @Resource
    private FxToolComponent fxToolComponent;
    @Resource
    private TrnmodService trnmodService;
    @Resource
    private TransactionHelpService transactionHelpService;
    @Resource
    private CbsmodService cbsmodService;
    @Resource
    private LiaallService liaallService;
    @Resource
    private SettleContext settleContext;
    @Autowired
    private PtaCacheService ptaCacheService;
    @Resource
    private BchService bchService;
    @Resource
    private XrtmodService xrtmodService;
    @Resource
    private SetmodService setmodService;
    @Resource
    private FxtpService fxtpService;
    @Autowired
    private PtyratMapper ptyratMapper;



    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDIA_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.UMDMOD_SERVICE, "N");
        serviceRegisterMaps.put(CommonContants.LIMMOD_SERVICE, "N");
        serviceRegisterMaps.put(CommonContants.BOPSET_RMB_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_SZSB_BOPREM_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_CFABOP_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.FANXIQIAN_SERVICE, "Y");
    }


    @Override
    protected void txnSave(FxtlopStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    @Override
    protected Map<String, String> txnCheck(FxtlopStoreBo transactionVo) {
        return fxtlopValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FxtlopStoreBo transactionVo) {
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        Fxd rec = transactionVo.getFxdgrp().getRec();
        bizInfoVo.setObjinr(rec.getInr());
        bizInfoVo.setOwnref(rec.getOwnref());
        bizInfoVo.setFxtyp(rec.getFxtyp());
        bizInfoVo.setAcc(rec.getAcc());
        bizInfoVo.setAcc2(rec.getAcc2());
        bizInfoVo.setDsp(rec.getDsp());
        bizInfoVo.setDsp2(rec.getDsp2());
        bizInfoVo.setTrdint(rec.getTrdint());
        bizInfoVo.setTrdout(rec.getTrdout());
        bizInfoVo.setTrnman(rec.getTrnman());
        bizInfoVo.setRat(rec.getRat());
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FxtlopStoreBo transactionVo) {
        return null;
    }

    @Override
    public FxtlopStoreBo init(FxtlopStoreBo fxtlopStoreBo) {
        fxtlopStoreBo.reset();
        fxtlopService.initFxtlop(fxtlopStoreBo);
        //fxtlopService.defaultFxtlop(fxtlopStoreBo);
        return fxtlopStoreBo;
    }

    @Override
    public void beforeSave(FxtlopStoreBo transactionVo) {
        transactionVo.reset();
        Fxdgrp fxdgrp = transactionVo.getFxdgrp();
        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(fxdgrp.getRec());
        transactionServiceSet.setBlk(fxdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        fxToolComponent.collectCbsMap(transactionServiceSet, "", fxdgrp.getCbs(), "max", "nom1", "opn1", "opn2");
        fxToolComponent.collectPtsList(transactionServiceSet, fxdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FxtlopStoreBo transactionVo) {
    }

    public void ruleMtabutSavN500(FxtlopStoreBo transactionVo){
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();

        fxToolComponent.getref(transactionVo.getFxdgrp());
        // 1. Create (rsp. update) Contract (incl. Texts)
        fxToolComponent.sav(transactionVo.getFxdgrp(), BigDecimal.ZERO);
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract ( trnmodVo);
        cbsSav(transactionVo, trnmodVo);
        // save documents
    }


    public void cbsSav(FxtlopStoreBo transactionVo, TrnmodVo trnmodVo) {
        cbsmodService.cbsTrnSetNew("TRN",trnmodVo.getTrn().getInr());
        //#cgl
        //平盘登记后产生4条CBB,MAX,NOM,OPN,OPN2
        //后续两个交易分别消OPN和OPN2
        String trninr = transactionHelpService.getTrninr();
        trnmodService.cbsTrnStore("MAXAMT","AMT1",Dates.today(),transactionVo.getFxdgrp().getCbs().getMax().getCur(),transactionVo.getFxdgrp().getCbs().getMax().getAmt(),MdaUtils.getI18NString("fxtlop","CT000000"),transactionVo.getFxdgrp().getRec(),trninr);
        trnmodService.cbsTrnStore("NOMAMT","AMT1",Dates.today(),transactionVo.getFxdgrp().getCbs().getNom1().getCur(),transactionVo.getFxdgrp().getCbs().getNom1().getAmt(),MdaUtils.getI18NString("fxtlop","CT000001"),transactionVo.getFxdgrp().getRec(),trninr);
        /**
         if FXDGRP\\REC\\FXTYP.is ( "LB" ) or FXDGRP\\REC\\FXTYP.is ( "LS" ) then

         **/
        /**
         if FXDGRP\\REC\\FXTYP.is ( "LB" ) then

         **/
        if( MdaUtils.compareTo(transactionVo.getFxdgrp().getRec().getFxtyp(),"LB")== 0 || MdaUtils.compareTo(transactionVo.getFxdgrp().getRec().getFxtyp(),"LS")== 0 )
        {
            trnmodService.cbsTrnStore("OPNAMT","AMT1",Dates.today(),transactionVo.getFxdgrp().getCbs().getMax().getCur(),transactionVo.getFxdgrp().getCbs().getMax().getAmt(),MdaUtils.getI18NString("fxtlop","CT000002"),transactionVo.getFxdgrp().getRec(),trninr);
        }
        if ( !MdaUtils.isEmpty( transactionHelpService.getTransactionStoreSet().getLiaall())) {
            liaallService.storeAll(transactionHelpService.getTransactionStoreSet().getLiaall());
        }
        //Platform.cbsTrnCommit();
    }

    public String getRef(FxtlopStoreVo fxtlopStoreVo) {
        fxtlopStoreVo.reset();
        PtsptaVo ptsptaVo = fxtlopStoreVo.getFxdgrp().getApl();
        if (ptsptaVo == null || StringUtils.isEmpty(ptsptaVo.getPtainr())) {
            return NULLSTR;
        }
        Pta pta = ptaCacheService.getPtaByInr(ptsptaVo.getPtainr());
        Bch wrkbch = new Bch();
        wrkbch.setInr(pta.getBranchinr());
        wrkbch = bchService.selectByPrimaryKey(wrkbch);
        if (Objects.isNull(wrkbch)) {
            return NULLSTR;
        }
        SettleContext.getUserSession().setWrkbch(wrkbch);
        Fxdgrp fxdgrp = fxtlopStoreVo.getFxdgrp();
        fxToolComponent.getref(fxdgrp);
        return fxdgrp.getRec().getOwnref();
    }

    public void defaultRateN1000(FxtlopStoreVo vo) {
        vo.reset();
        String rmbRat = NULLSTR;
        BigDecimal buyRat = null;
        BigDecimal selRat = null;
        BigDecimal midRat = null;
        BigDecimal buy1Rat = null;
        String selTyp = NULLSTR;
        BigDecimal selPreRat = null;
        String buyTyp = NULLSTR;
        BigDecimal buyPreRat = null;
        String setCur = NULLSTR;
        BigDecimal setAmt = BigDecimal.ZERO;
        BigDecimal favRat = BigDecimal.ZERO;
        Cbb max = vo.getFxdgrp().getCbs().getMax();
        Fxd rec = vo.getFxdgrp().getRec();
        Pts pts = vo.getFxdgrp().getApl().getPts();
        if (MdaUtils.isEmpty(vo.getFxtp().getQuoflg())
                && !MdaUtils.isEmpty(rec.getFxtyp()) && !MdaUtils.isEmpty(pts.getPtyinr())) {
//            Systems.setEnabled(vo.getFxdgrp().getRec(), "rat");
            Argument<String> rmbRatBox = new Argument<String>();
            Argument<BigDecimal> buyRatBox = new Argument<BigDecimal>();
            Argument<BigDecimal> selRatBox = new Argument<BigDecimal>();
            Argument<BigDecimal> midRatBox = new Argument<BigDecimal>();
            Argument<BigDecimal> buy1RatBox = new Argument<BigDecimal>();
            xrtmodService.getratCebTim(max.getCur(), xrtmodService.sysiso(), Dates.today(), rmbRatBox, buyRatBox, selRatBox, midRatBox, buy1RatBox);
            rmbRat = rmbRatBox.value;
            buyRat = buyRatBox.value;
            selRat = selRatBox.value;
            midRat = midRatBox.value;
            buy1Rat = buy1RatBox.value;
            Ptyrat ptyrat = ptyratMapper.getPtyratByPtyinrAndCur(pts.getPtyinr(), max.getCur());
            if (Objects.nonNull(ptyrat)) {
                selTyp = ptyrat.getSeltyp();
                selPreRat = ptyrat.getBuyrat();
                buyTyp = ptyrat.getBuytyp();
                buyPreRat = ptyrat.getBuyrat();
            }
            // 如果没有优惠，则根据配置文件来进行优惠
            setCur = max.getCur();
            SetmodVo setmodVo = buildSetmodVo(vo);
            setAmt = setmodService.getSettlementAmount(setmodVo, setCur);
            // 默认都是按照点数优惠
            if (MdaUtils.compareTo(setAmt, 0) > 0) {
                buyTyp = "1";
                selTyp = "1";
               /* TODO setusd.ini  未提供 暂时注释
               favRat = vo.getFxtp().getFavorRatePoint(setCur, setAmt);
                if (MdaUtils.compareTo(favRat, buyPreRat) > 0) {
                    buyPreRat = favRat;
                }
                if (MdaUtils.compareTo(favRat, selPreRat) > 0) {
                    selPreRat = favRat;
                }*/
            }
            if (!rec.isModified("rat")) {
                if (MdaUtils.compareTo(Strings.mid(rec.getFxtyp(), 2, 1), "B") == 0) {
                    if (MdaUtils.compareTo(vo.getFxtp().getCshflg(), "2") == 0) {
                        rec.setRat(buy1Rat);
                    } else {
                        switch (buyTyp) {
                            case "1":
                                rec.setRat(Decimals.add(buyRat, Decimals.div(buyPreRat, 10000)));
                                break;
                            case "2":
                                rec.setRat(Decimals.add(buyRat, Decimals.div(Decimals.mul(buyRat, buyPreRat), 100)));
                                break;
                            default:
                                rec.setRat(buyRat);
                        }
                    }
                } else {
                    switch (Strings.trim(selTyp)) {
                        case "1":
                            rec.setRat(Decimals.sub(selRat, Decimals.div(selPreRat, 10000)));
                            break;
                        case "2":
                            rec.setRat(Decimals.sub(selRat, Decimals.div(Decimals.mul(selRat, selPreRat), 100)));
                            break;
                        default:
                            rec.setRat(selRat);
                    }
                }
            }
            rec.setRat(Numerics.round(rec.getRat(), 4));
            rec.setMidrat(Numerics.round(midRat, 4));
        } else {
//            Systems.resetEnabled(rec, "rat");
            rec.setRat(BigDecimal.ZERO);
            rec.setMidrat(BigDecimal.ZERO);
        }
    }

    private SetmodVo buildSetmodVo(FxtlopStoreVo vo) {
        SetmodVo setmodVo = new SetmodVo();
        Map cbsMap = new HashMap();
        Cbb max = vo.getFxdgrp().getCbs().getMax();
        cbsMap.put("MAX", max);
        setmodVo.setCbsMap(cbsMap);
        List<PtsptaVo> ptsList = new ArrayList();
        PtsptaVo apl = vo.getFxdgrp().getApl();
        ptsList.add(apl);
        setmodVo.setPtsList(ptsList);
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        setmodVo.setRec(bizInfoVo);
        return setmodVo;
    }

    public void defaultCalculateN1100(FxtlopStoreVo fxtlopStoreVo) {
        fxtlopStoreVo.reset();
        fxtpService.defaultCalculateN1100(fxtlopStoreVo.getFxdgrp(), fxtlopStoreVo.getFxtp());
    }
}
