package com.brilliance.isc.funds.transfer.transaction;

import com.brilliance.isc.funds.bo.funds.CrtcriStoreBo;
import com.brilliance.isc.funds.vo.funds.CrtcriStoreVo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;

/**
 * @program: isc-funds
 * @ClassName: CrtcriStructMapper
 * @description: 汇入清算 Crtcri Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 * @author: huangshunlin
 * @create: 2024-10-16 20:30
 */
@Mapper(componentModel = "spring")
public interface CrtcriStructMapper extends IBaseStructMapper<CrtcriStoreBo, CrtcriStoreVo> {
    /**
     * 此行语句一定要写，StructMapper需要
     */
    CrtcriStructMapper INSTANCE = Mappers.getMapper(CrtcriStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    CrtcriStoreBo voTransferToBo(CrtcriStoreVo vo);

    /**
     * Vo追加更新至Bo
     *
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({

    })
    void voUpdateToBo(CrtcriStoreVo vo, @MappingTarget CrtcriStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    CrtcriStoreVo boTransferToVo(CrtcriStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(CrtcriStoreBo bo, @MappingTarget CrtcriStoreVo vo);
}

