package com.brilliance.isc.funds.transfer.transaction;

import com.brilliance.isc.funds.bo.funds.CrtctfStoreBo;
import com.brilliance.isc.funds.vo.funds.CrtctfStoreVo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;

/**
 * @program: isc-funds
 * @ClassName: CrtctfStructMapper
 * @description: 转汇清算 Crtctf Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 * @author: huangshunlin
 * @create: 2024-10-16 21:44
 */
@Mapper(componentModel = "spring")
public interface CrtctfStructMapper extends IBaseStructMapper<CrtctfStoreBo, CrtctfStoreVo> {
    /**
     * 此行语句一定要写，StructMapper需要
     */
    CrtctfStructMapper INSTANCE = Mappers.getMapper(CrtctfStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    CrtctfStoreBo voTransferToBo(CrtctfStoreVo vo);

    /**
     * Vo追加更新至Bo
     *
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({

    })
    void voUpdateToBo(CrtctfStoreVo vo, @MappingTarget CrtctfStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    CrtctfStoreVo boTransferToVo(CrtctfStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(CrtctfStoreBo bo, @MappingTarget CrtctfStoreVo vo);
}

