package com.brilliance.isc.funds.transfer.transaction;

import com.brilliance.isc.funds.bo.funds.FxtlopStoreBo;
import com.brilliance.isc.funds.vo.funds.FxtlopStoreVo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;

/**
 * @program: isc-funds
 * @ClassName: FxtlopStructMapper
 * @description: 结售汇平盘登记转换Fxtlop Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 * @author: wangweidong
 * @create: 2024-10-08 16:27
 */
@Mapper(componentModel = "spring")
public interface FxtlopStructMapper extends IBaseStructMapper<FxtlopStoreBo, FxtlopStoreVo> {
    /**
     * 此行语句一定要写，StructMapper需要
     */
    FxtlopStructMapper INSTANCE = Mappers.getMapper(FxtlopStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    FxtlopStoreBo voTransferToBo(FxtlopStoreVo vo);

    /**
     * Vo追加更新至Bo
     *
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({

    })
    void voUpdateToBo(FxtlopStoreVo vo, @MappingTarget FxtlopStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    FxtlopStoreVo boTransferToVo(FxtlopStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(FxtlopStoreBo bo, @MappingTarget FxtlopStoreVo vo);
}

